@@ -24,14 +24,8 @@
 
 #include "plugin.h"
 #include "connectioncontainer.h"
-
 #include "utils.h"
 
-#include <assert.h>
-#include <rpc.h>
-#include <rpcndr.h>
-#include <objidl.h>
-
 using namespace std;
 
 #ifdef __MINGW32__
