@@ -94,6 +94,7 @@ static LRESULT CALLBACK VLCInPlaceClassWndProc(HWND hWnd, UINT uMsg, WPARAM wPar
 
 VLCPluginClass::VLCPluginClass(LONG *p_class_ref, HINSTANCE hInstance, REFCLSID rclsid) :
     _p_class_ref(p_class_ref),
+    _class_ref(0),
     _hinstance(hInstance),
     _classid(rclsid),
     _inplace_picture(NULL)
@@ -166,18 +167,19 @@ STDMETHODIMP VLCPluginClass::QueryInterface(REFIID riid, void **ppv)
 
 STDMETHODIMP_(ULONG) VLCPluginClass::AddRef(void)
 {
-    return InterlockedIncrement(_p_class_ref);
+    InterlockedIncrement(_p_class_ref);
+    return ++_class_ref;
 };
 
 STDMETHODIMP_(ULONG) VLCPluginClass::Release(void)
 {
-    ULONG refcount = InterlockedDecrement(_p_class_ref);
-    if( 0 == refcount )
+    InterlockedDecrement(_p_class_ref);
+    if( 0 == (--_class_ref) )
     {
         delete this;
         return 0;
     }
-    return refcount;
+    return _class_ref;
 };
 
 STDMETHODIMP VLCPluginClass::CreateInstance(LPUNKNOWN pUnkOuter, REFIID riid, void **ppv)
@@ -282,6 +284,14 @@ VLCPlugin::VLCPlugin(VLCPluginClass *p_class, LPUNKNOWN pUnkOuter) :
 
 VLCPlugin::~VLCPlugin()
 {
+    if( vlc_player::is_open() )
+    {
+        if( isPlaying() )
+            playlist_stop();
+
+        player_unregister_events();
+    }
+
     delete vlcSupportErrorInfo;
     delete vlcOleObject;
     delete vlcDataObject;
@@ -304,14 +314,6 @@ VLCPlugin::~VLCPlugin()
     SysFreeString(_bstr_mrl);
     SysFreeString(_bstr_baseurl);
 
-    if( vlc_player::is_open() )
-    {
-        if( isPlaying() )
-            playlist_stop();
-
-        player_unregister_events();
-    }
-
     if( _p_libvlc )  { libvlc_release(_p_libvlc); _p_libvlc=NULL; }
 
     _p_class->Release();
@@ -399,7 +401,6 @@ HRESULT VLCPlugin::onInit(void)
         // initialize persistable properties
         set_autoplay(true);
         _b_autoloop   = FALSE;
-        set_show_toolbar(false);
         _bstr_baseurl = NULL;
         _bstr_mrl     = NULL;
         _b_visible    = TRUE;
@@ -522,11 +523,6 @@ void VLCPlugin::initVLC()
     ppsz_argv[ppsz_argc++] = "--intf=dummy";
     ppsz_argv[ppsz_argc++] = "--no-video-title-show";
 
-
-    // loop mode is a configuration option only
-    if( _b_autoloop )
-        ppsz_argv[ppsz_argc++] = "--loop";
-
     _p_libvlc = libvlc_new(ppsz_argc, ppsz_argv);
     if( !_p_libvlc )
         return;
@@ -534,7 +530,11 @@ void VLCPlugin::initVLC()
     if( !vlc_player::open(_p_libvlc) )
         return;
 
-    set_player_window();
+    vlc_player::set_mode(_b_autoloop ? libvlc_playback_mode_loop :
+                                       libvlc_playback_mode_default);
+
+    // register player events
+    player_register_events();
 
     // initial playlist item
     if( SysStringLen(_bstr_mrl) > 0 )
@@ -580,6 +580,18 @@ void VLCPlugin::initVLC()
             CoTaskMemFree(psz_mrl);
         }
     }
+
+    if( !isInPlaceActive()  )
+    {
+        LPOLECLIENTSITE pClientSite;
+        if( SUCCEEDED(vlcOleObject->GetClientSite(&pClientSite)) && (NULL != pClientSite) )
+        {
+            vlcOleObject->DoVerb(OLEIVERB_INPLACEACTIVATE, NULL, pClientSite, 0, NULL, NULL);
+            pClientSite->Release();
+        }
+    }
+
+    set_player_window();
 };
 
 void VLCPlugin::setErrorInfo(REFIID riid, const char *description)
@@ -1260,6 +1272,26 @@ void VLCPlugin::on_player_action(vlc_player_action_e pa)
     }
 }
 
+static vlcplugin_event_t vlcevents[] = {
+    //{ libvlc_MediaPlayerMediaChanged, handle_input_state_event },         // unused
+    { libvlc_MediaPlayerNothingSpecial, handle_input_state_event },
+    { libvlc_MediaPlayerOpening, handle_input_state_event },
+    { libvlc_MediaPlayerBuffering, handle_input_state_event },
+    { libvlc_MediaPlayerPlaying, handle_input_state_event },
+    { libvlc_MediaPlayerPaused, handle_input_state_event },
+    { libvlc_MediaPlayerStopped, handle_input_state_event },
+    { libvlc_MediaPlayerForward, handle_input_state_event },
+    { libvlc_MediaPlayerBackward, handle_input_state_event },
+    { libvlc_MediaPlayerEndReached, handle_input_state_event },
+    { libvlc_MediaPlayerEncounteredError, handle_input_state_event },
+    { libvlc_MediaPlayerTimeChanged, handle_time_changed_event },
+    { libvlc_MediaPlayerPositionChanged, handle_position_changed_event },
+    { libvlc_MediaPlayerSeekableChanged, handle_seekable_changed_event },
+    { libvlc_MediaPlayerPausableChanged, handle_pausable_changed_event },
+    //{ libvlc_MediaPlayerTitleChanged, handle_input_state_event },         // unused
+    //{ libvlc_MediaPlayerLengthChanged, handle_input_state_event },        // unused
+};
+
 void VLCPlugin::player_register_events()
 {
     libvlc_event_manager_t *eventManager = NULL;
@@ -1267,35 +1299,13 @@ void VLCPlugin::player_register_events()
 
     eventManager = libvlc_media_player_event_manager(vlc_player::get_mp());
     if(eventManager) {
-        libvlc_event_attach(eventManager, libvlc_MediaPlayerNothingSpecial,
-                            handle_input_state_event, this);
-        libvlc_event_attach(eventManager, libvlc_MediaPlayerOpening,
-                            handle_input_state_event, this);
-        libvlc_event_attach(eventManager, libvlc_MediaPlayerBuffering,
-                            handle_input_state_event, this);
-        libvlc_event_attach(eventManager, libvlc_MediaPlayerPlaying,
-                            handle_input_state_event, this);
-        libvlc_event_attach(eventManager, libvlc_MediaPlayerPaused,
-                            handle_input_state_event, this);
-        libvlc_event_attach(eventManager, libvlc_MediaPlayerStopped,
-                            handle_input_state_event, this);
-        libvlc_event_attach(eventManager, libvlc_MediaPlayerForward,
-                            handle_input_state_event, this);
-        libvlc_event_attach(eventManager, libvlc_MediaPlayerBackward,
-                            handle_input_state_event, this);
-        libvlc_event_attach(eventManager, libvlc_MediaPlayerEndReached,
-                            handle_input_state_event, this);
-        libvlc_event_attach(eventManager, libvlc_MediaPlayerEncounteredError,
-                            handle_input_state_event, this);
-
-        libvlc_event_attach(eventManager, libvlc_MediaPlayerTimeChanged,
-                            handle_time_changed_event, this);
-        libvlc_event_attach(eventManager, libvlc_MediaPlayerPositionChanged,
-                            handle_position_changed_event, this);
-        libvlc_event_attach(eventManager, libvlc_MediaPlayerSeekableChanged,
-                            handle_seekable_changed_event, this);
-        libvlc_event_attach(eventManager, libvlc_MediaPlayerPausableChanged,
-                            handle_pausable_changed_event, this);
+        /* attach all libvlc events we care about */
+        for( size_t i = 0; i < ARRAY_SIZE(vlcevents); i++ )
+        {
+            libvlc_event_attach( eventManager, vlcevents[i].libvlc_type,
+                                               vlcevents[i].libvlc_callback,
+                                               this );
+        }
     }
 }
 
@@ -1306,34 +1316,12 @@ void VLCPlugin::player_unregister_events()
 
     eventManager = libvlc_media_player_event_manager(vlc_player::get_mp());
     if(eventManager) {
-        libvlc_event_detach(eventManager, libvlc_MediaPlayerNothingSpecial,
-                            handle_input_state_event, this);
-        libvlc_event_detach(eventManager, libvlc_MediaPlayerOpening,
-                            handle_input_state_event, this);
-        libvlc_event_detach(eventManager, libvlc_MediaPlayerBuffering,
-                            handle_input_state_event, this);
-        libvlc_event_detach(eventManager, libvlc_MediaPlayerPlaying,
-                            handle_input_state_event, this);
-        libvlc_event_detach(eventManager, libvlc_MediaPlayerPaused,
-                            handle_input_state_event, this);
-        libvlc_event_detach(eventManager, libvlc_MediaPlayerStopped,
-                            handle_input_state_event, this);
-        libvlc_event_detach(eventManager, libvlc_MediaPlayerForward,
-                            handle_input_state_event, this);
-        libvlc_event_detach(eventManager, libvlc_MediaPlayerBackward,
-                            handle_input_state_event, this);
-        libvlc_event_detach(eventManager, libvlc_MediaPlayerEndReached,
-                            handle_input_state_event, this);
-        libvlc_event_detach(eventManager, libvlc_MediaPlayerEncounteredError,
-                            handle_input_state_event, this);
-
-        libvlc_event_detach(eventManager, libvlc_MediaPlayerTimeChanged,
-                            handle_time_changed_event, this);
-        libvlc_event_detach(eventManager, libvlc_MediaPlayerPositionChanged,
-                            handle_position_changed_event, this);
-        libvlc_event_detach(eventManager, libvlc_MediaPlayerSeekableChanged,
-                            handle_seekable_changed_event, this);
-        libvlc_event_detach(eventManager, libvlc_MediaPlayerPausableChanged,
-                            handle_pausable_changed_event, this);
+        /* detach all libvlc events we cared about */
+        for( size_t i = 0; i < ARRAY_SIZE(vlcevents); i++ )
+        {
+            libvlc_event_detach( eventManager, vlcevents[i].libvlc_type,
+                                               vlcevents[i].libvlc_callback,
+                                               this );
+        }
     }
 }
