@@ -24,22 +24,26 @@
 #ifndef __PLUGIN_H__
 #define __PLUGIN_H__
 
-#include "config.h"
+#ifdef HAVE_CONFIG
+# include "config.h"
+#endif
 
-#include <ole2.h>
 #include <olectl.h>
 
-#include <vlc/vlc.h>
-
 #include "../common/win32_fullscreen.h"
-#include "../common/vlc_player_options.h"
-#include "../common/vlc_player.h"
 
 extern "C" const GUID CLSID_VLCPlugin;
 extern "C" const GUID CLSID_VLCPlugin2;
 extern "C" const GUID LIBID_AXVLC;
 extern "C" const GUID DIID_DVLCEvents;
 
+#define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))
+
+typedef struct {
+    const libvlc_event_type_t libvlc_type; /* libvlc event type */
+    libvlc_callback_t libvlc_callback;     /* libvlc callback function */
+} vlcplugin_event_t;
+
 class VLCPluginClass : public IClassFactory
 {
 
@@ -70,13 +74,14 @@ protected:
 private:
 
     LPLONG      _p_class_ref;
+    ULONG       _class_ref;
     HINSTANCE   _hinstance;
     CLSID       _classid;
     ATOM        _inplace_wndclass_atom;
     LPPICTURE   _inplace_picture;
 };
 
-struct VLCPlugin
+class VLCPlugin
     : public IUnknown, private vlc_player_options, private vlc_player
 {
 public:
@@ -124,6 +129,10 @@ public:
     inline void setAutoLoop(BOOL autoloop)
     {
         _b_autoloop = autoloop;
+        if( vlc_player::is_open() ){
+            vlc_player::set_mode(autoloop ? libvlc_playback_mode_loop :
+                                            libvlc_playback_mode_default);
+        }
         setDirty(TRUE);
     };
     inline BOOL getAutoLoop(void) { return _b_autoloop;};
