@@ -33,7 +33,7 @@
 #include "vlccontrol2.h"
 #include "vlccontrol.h"
 
-#include "position.h"
+#include "../common/position.h"
 
 // ---------
 
@@ -557,12 +557,18 @@ STDMETHODIMP VLCPlaylist::playItem(long item)
     return S_OK;
 };
 
-STDMETHODIMP VLCPlaylist::togglePause()
+STDMETHODIMP VLCPlaylist::pause()
 {
     Instance()->get_player().pause();
     return S_OK;
 };
 
+STDMETHODIMP VLCPlaylist::togglePause()
+{
+    Instance()->get_player().togglePause();
+    return S_OK;
+};
+
 STDMETHODIMP VLCPlaylist::stop()
 {
     Instance()->get_player().stop();
