@@ -27,7 +27,6 @@
 
 #include "axvlc_idl.h"
 
-#include <vlc/libvlc.h>
 #include <ole2.h>
 
 class VLCInterfaceBase {
@@ -67,6 +66,8 @@ public:
     VLCInterface(VLCPlugin *p): Base(p) { }
     VLCPlugin *Instance() const { return Base::Instance(); }
 
+    virtual ~VLCInterface() { }
+
     HRESULT getVLC(libvlc_instance_t **pp) const { return Base::getVLC(pp); }
     HRESULT getMD(libvlc_media_player_t **pp) const { return Base::getMD(pp); }
 
@@ -170,6 +171,7 @@ class VLCMarquee: public VLCInterface<VLCMarquee,IVLCMarquee>
 {
 public:
     VLCMarquee(VLCPlugin *p): VLCInterface<VLCMarquee,IVLCMarquee>(p) { }
+    virtual ~VLCMarquee() { }
 
     // IVLCMarquee methods
     STDMETHODIMP enable()  { return do_put_int(libvlc_marquee_Enable, true); }
@@ -203,6 +205,7 @@ class VLCLogo: public VLCInterface<VLCLogo,IVLCLogo>
 {
 public:
     VLCLogo(VLCPlugin *p): VLCInterface<VLCLogo,IVLCLogo>(p) { }
+    virtual ~VLCLogo() { }
 
     STDMETHODIMP enable()  { return do_put_int(libvlc_logo_enable, true); }
     STDMETHODIMP disable() { return do_put_int(libvlc_logo_enable, false); }
@@ -236,6 +239,7 @@ class VLCDeinterlace: public VLCInterface<VLCDeinterlace,IVLCDeinterlace>
 public:
     VLCDeinterlace(VLCPlugin *p):
         VLCInterface<VLCDeinterlace,IVLCDeinterlace>(p) { }
+    virtual ~VLCDeinterlace() { }
 
     STDMETHODIMP enable(BSTR val);
     STDMETHODIMP disable();
@@ -267,6 +271,7 @@ public:
     STDMETHODIMP add(BSTR, VARIANT, VARIANT, long*);
     STDMETHODIMP play();
     STDMETHODIMP playItem(long);
+    STDMETHODIMP pause();
     STDMETHODIMP togglePause();
     STDMETHODIMP stop();
     STDMETHODIMP next();
@@ -298,9 +303,9 @@ public:
         _p_vlcmarquee(new VLCMarquee(p)), _p_vlclogo(new VLCLogo(p)),
         _p_vlcdeint(new VLCDeinterlace(p)) { }
     virtual ~VLCVideo() {
-        delete _p_vlcmarquee;
-        delete _p_vlclogo;
-        delete _p_vlcdeint;
+        _p_vlcmarquee->Release();
+        _p_vlclogo->Release();
+        _p_vlcdeint->Release();
     }
 
     // IVLCVideo methods
