@@ -511,7 +511,7 @@ void VLCControlsWnd::SyncVolumeSliderWithVLCVolume()
         vlc_player& vp = *VP();
         unsigned int vol = vp.get_volume();
         const LRESULT SliderPos = SendMessage(hVolumeSlider, (UINT) TBM_GETPOS, 0, 0);
-        if(SliderPos!=vol)
+        if((UINT)SliderPos!=vol)
             SendMessage(hVolumeSlider, (UINT) TBM_SETPOS, (WPARAM) TRUE, (LPARAM) vol);
 
         bool muted = vp.is_muted();
@@ -898,6 +898,11 @@ LRESULT CALLBACK VLCFullScreenWnd::FSWndWindowProc(HWND hWnd, UINT uMsg, WPARAM
             }
             break;
         }
+        case WM_KEYDOWN: {
+            if (fs_data)
+                fs_data->_WindowsManager->OnKeyDownEvent(wParam);
+            break;
+        }
         default:
             return DefWindowProc(hWnd, uMsg, wParam, lParam);
     }
@@ -1064,6 +1069,16 @@ bool VLCWindowsManager::IsFullScreen()
     return 0!=_FSWnd && 0!=_HolderWnd && GetParent(_HolderWnd->hWnd())==_FSWnd->getHWND();
 }
 
+void VLCWindowsManager::OnKeyDownEvent(UINT uKeyMsg)
+{
+    switch(uKeyMsg){
+        case VK_ESCAPE:
+        case 'F':
+            EndFullScreen();
+            break;
+    }
+}
+
 void VLCWindowsManager::OnMouseEvent(UINT uMouseMsg)
 {
     switch(uMouseMsg){
