@@ -57,8 +57,12 @@
 #endif
 
 #include <cstring>
+#include "../common.h"
 #include "../vlcshell.h"
 
+#ifdef USE_GTK
+# include <gtk/gtk.h>
+#endif
 /*
  * Define PLUGIN_TRACE to have the wrapper functions print
  * messages to stderr whenever they are called.
@@ -323,27 +327,6 @@ void NPN_ReloadPlugins(NPBool reloadPages)
 #endif
 }
 
-#ifdef OJI
-JRIEnv* NPN_GetJavaEnv()
-{
-#if (((NP_VERSION_MAJOR << 8) + NP_VERSION_MINOR) < 20)
-    return CallNPN_GetJavaEnvProc(gNetscapeFuncs.getJavaEnv);
-#else
-    return (*gNetscapeFuncs.getJavaEnv);
-#endif
-}
-
-jref NPN_GetJavaPeer(NPP instance)
-{
-#if (((NP_VERSION_MAJOR << 8) + NP_VERSION_MINOR) < 20)
-    return CallNPN_GetJavaPeerProc(gNetscapeFuncs.getJavaPeer,
-                       instance);
-#else
-    return (*gNetscapeFuncs.getJavaPeer)(instance);
-#endif
-}
-#endif
-
 void
 NPN_InvalidateRect(NPP instance, NPRect *invalidRect)
 {
@@ -680,11 +663,9 @@ NPError Private_DestroyStream(NPP instance, NPStream* stream, NPError reason);
 void Private_URLNotify(NPP instance, const char* url,
                        NPReason reason, void* notifyData);
 void Private_Print(NPP instance, NPPrint* platformPrint);
+int16_t Private_HandleEvent(NPP instance, NPEvent *event);
 NPError Private_GetValue(NPP instance, NPPVariable variable, void *r_value);
 NPError Private_SetValue(NPP instance, NPNVariable variable, void *r_value);
-#ifdef OJI
-JRIGlobalRef Private_GetJavaClass(void);
-#endif
 
 /* function implementations */
 NPError
@@ -775,6 +756,13 @@ Private_Print(NPP instance, NPPrint* platformPrint)
     NPP_Print(instance, platformPrint);
 }
 
+int16_t
+Private_HandleEvent(NPP instance, NPEvent *event)
+{
+    PLUGINDEBUGSTR("HandleEvent");
+    NPP_HandleEvent(instance, event);
+}
+
 NPError
 Private_GetValue(NPP instance, NPPVariable variable, void *r_value)
 {
@@ -789,19 +777,6 @@ Private_SetValue(NPP instance, NPNVariable variable, void *r_value)
     return NPP_SetValue(instance, variable, r_value);
 }
 
-#ifdef OJI
-JRIGlobalRef
-Private_GetJavaClass(void)
-{
-    jref clazz = NPP_GetJavaClass();
-    if (clazz) {
-    JRIEnv* env = NPN_GetJavaEnv();
-    return JRI_NewGlobalRef(env, clazz);
-    }
-    return NULL;
-}
-#endif
-
 /*********************************************************************** 
  *
  * These functions are located automagically by netscape.
@@ -872,7 +847,7 @@ NP_Initialize(NPNetscapeFuncs* nsTable, NPPluginFuncs* pluginFuncs)
             err = NPERR_INCOMPATIBLE_VERSION_ERROR;
         if (nsTable->size < ((char *)&nsTable->posturlnotify - (char *)nsTable))
             err = NPERR_INVALID_FUNCTABLE_ERROR;
-        if (pluginFuncs->size < sizeof(NPPluginFuncs))
+        if (pluginFuncs->size < (char *)&pluginFuncs->setvalue - (char *)pluginFuncs)
             err = NPERR_INVALID_FUNCTABLE_ERROR;
     }
 
@@ -904,13 +879,6 @@ NP_Initialize(NPNetscapeFuncs* nsTable, NPPluginFuncs* pluginFuncs)
 #if (((NP_VERSION_MAJOR << 8) + NP_VERSION_MINOR) >= 20)
         gNetscapeFuncs.pluginthreadasynccall =
             nsTable->pluginthreadasynccall;
-#endif
-#ifdef OJI
-        if( minor >= NPVERS_HAS_LIVECONNECT )
-        {
-            gNetscapeFuncs.getJavaEnv    = nsTable->getJavaEnv;
-            gNetscapeFuncs.getJavaPeer   = nsTable->getJavaPeer;
-        }
 #endif
         gNetscapeFuncs.getvalue      = nsTable->getvalue;
         gNetscapeFuncs.setvalue      = nsTable->setvalue;
@@ -1004,6 +972,7 @@ NP_Initialize(NPNetscapeFuncs* nsTable, NPPluginFuncs* pluginFuncs)
         pluginFuncs->writeready = NewNPP_WriteReadyProc(Private_WriteReady);
         pluginFuncs->write      = NewNPP_WriteProc(Private_Write);
         pluginFuncs->print      = NewNPP_PrintProc(Private_Print);
+        pluginFuncs->event      = NewNPP_HandleEventProc(Private_HandleEvent);
         pluginFuncs->getvalue   = NewNPP_GetValueProc(Private_GetValue);
         pluginFuncs->setvalue   = NewNPP_SetValueProc(Private_SetValue);
 #else
@@ -1016,10 +985,10 @@ NP_Initialize(NPNetscapeFuncs* nsTable, NPPluginFuncs* pluginFuncs)
         pluginFuncs->writeready = (NPP_WriteReadyProcPtr)(Private_WriteReady);
         pluginFuncs->write      = (NPP_WriteProcPtr)(Private_Write);
         pluginFuncs->print      = (NPP_PrintProcPtr)(Private_Print);
+        pluginFuncs->event      = (NPP_HandleEventProcPtr)(Private_HandleEvent);
         pluginFuncs->getvalue   = (NPP_GetValueProcPtr)(Private_GetValue);
         pluginFuncs->setvalue   = (NPP_SetValueProcPtr)(Private_SetValue);
 #endif
-        pluginFuncs->event      = NULL;
         if( minor >= NPVERS_HAS_NOTIFICATION )
         {
 #if (((NP_VERSION_MAJOR << 8) + NP_VERSION_MINOR) < 20)
@@ -1028,14 +997,7 @@ NP_Initialize(NPNetscapeFuncs* nsTable, NPPluginFuncs* pluginFuncs)
             pluginFuncs->urlnotify = (NPP_URLNotifyProcPtr)(Private_URLNotify);
 #endif
         }
-#ifdef OJI
-        if( minor >= NPVERS_HAS_LIVECONNECT )
-            pluginFuncs->javaClass  = Private_GetJavaClass();
-        else
-            pluginFuncs->javaClass = NULL;
-#else
         pluginFuncs->javaClass = NULL;
-#endif
 
         err = NPP_Initialize();
     }
