@@ -23,6 +23,7 @@
 
 #include "vlcplugin_gtk.h"
 #include <gdk/gdkx.h>
+#include <gdk/gdkkeysyms.h>
 #include <cstring>
 
 static uint32_t get_xid(GtkWidget *widget)
@@ -358,6 +359,22 @@ static void fullscreen_win_visibility_handler(GtkWidget *widget, gpointer user_d
     plugin->do_set_fullscreen(gtk_widget_get_visible(widget));
 }
 
+static gboolean fullscreen_win_keypress_handler(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
+{
+    VlcPluginGtk *plugin = (VlcPluginGtk *) user_data;
+    switch (event->keyval)
+    {
+    case GDK_KEY_space:
+        plugin->playlist_togglePause();
+        return True;
+    case GDK_KEY_Escape:
+        plugin->set_fullscreen(false);
+        return True;
+    default:
+        return False;
+    }
+}
+
 void VlcPluginGtk::update_controls()
 {
     if (get_player().is_open()) {
@@ -445,6 +462,7 @@ bool VlcPluginGtk::create_windows()
     g_signal_connect(G_OBJECT(fullscreen_win), "delete-event", G_CALLBACK(gtk_widget_hide_on_delete), this);
     g_signal_connect(G_OBJECT(fullscreen_win), "show", G_CALLBACK(fullscreen_win_visibility_handler), this);
     g_signal_connect(G_OBJECT(fullscreen_win), "hide", G_CALLBACK(fullscreen_win_visibility_handler), this);
+    g_signal_connect(G_OBJECT(fullscreen_win), "key_press_event", G_CALLBACK(fullscreen_win_keypress_handler), this);
 
     /* actual video window */
     /* libvlc is handed this window's xid. A raw X window is used because
@@ -491,7 +509,7 @@ bool VlcPluginGtk::create_windows()
     gtk_container_add(GTK_CONTAINER(toolitem), time_slider);
     gtk_tool_item_set_expand(toolitem, true);
     gtk_toolbar_insert(GTK_TOOLBAR(toolbar), toolitem, -1);
-    
+
     /* volume slider */
     toolitem = gtk_tool_item_new();
     vol_slider = gtk_hscale_new_with_range(0, 200, 10);
@@ -513,6 +531,7 @@ bool VlcPluginGtk::resize_windows()
     req.width = npwindow.width;
     req.height = npwindow.height;
     gtk_widget_size_request(parent, &req);
+    return true;
 }
 
 bool VlcPluginGtk::destroy_windows()
@@ -529,4 +548,5 @@ bool VlcPluginGtk::destroy_windows()
     /* free colors */
     Colormap colormap = DefaultColormap(display, DefaultScreen(display));
     XFreeColors(display, colormap, &bg_color.pixel, 1, 0);
+    return true;
 }
