@@ -34,9 +34,83 @@
 #include <string.h>
 #include <stdlib.h>
 
-#include "vlcplugin.h"
+#include "common.h"
 #include "vlcshell.h"
+#include "vlcplugin.h"
 
+static char mimetype[] =
+    /* MPEG-1 and MPEG-2 */
+    "audio/mpeg:mp2,mp3,mpga,mpega:MPEG audio;"
+    "audio/x-mpeg:mp2,mp3,mpga,mpega:MPEG audio;"
+    "video/mpeg:mpg,mpeg,mpe:MPEG video;"
+    "video/x-mpeg:mpg,mpeg,mpe:MPEG video;"
+    "video/mpeg-system:mpg,mpeg,mpe,vob:MPEG video;"
+    "video/x-mpeg-system:mpg,mpeg,mpe,vob:MPEG video;"
+    /* MPEG-4 */
+    "audio/mp4:aac,mp4,mpg4:MPEG-4 audio;"
+    "audio/x-m4a:m4a:MPEG-4 audio;"
+    "audio/m4a:m4a:MPEG-4 audio;"
+    /* MPEG-4 ASP */
+    "video/mp4:mp4,mpg4:MPEG-4 video;"
+    "application/mpeg4-iod:mp4,mpg4:MPEG-4 video;"
+    "application/mpeg4-muxcodetable:mp4,mpg4:MPEG-4 video;"
+    "video/x-m4v:m4v:MPEG-4 video;"
+    /* AVI */
+    "video/x-msvideo:avi:AVI video;"
+    /* QuickTime */
+    "video/quicktime:mov,qt:QuickTime video;"
+    /* OGG */
+    "application/ogg:ogg:Ogg stream;"
+    "video/ogg:ogv:Ogg video;"
+    "audio/ogg:oga:Ogg audio;"
+    "application/x-ogg:ogg:Ogg stream;"
+    /* Opus */
+    "audio/ogg;codecs=opus:opus:Opus audio;"
+    /* VLC */
+    "application/x-vlc-plugin::VLC plug-in;"
+    /* Windows Media */
+    "video/x-ms-asf-plugin:asf,asx:Windows Media Video;"
+    "video/x-ms-asf:asf,asx:Windows Media Video;"
+    "application/x-mplayer2::Windows Media;"
+    "video/x-ms-wmv:wmv:Windows Media;"
+    "video/x-ms-wvx:wvx:Windows Media Video;"
+    "audio/x-ms-wma:wma:Windows Media Audio;"
+    /* Google VLC */
+    "application/x-google-vlc-plugin::Google VLC plug-in;"
+    /* WAV audio */
+    "audio/wav:wav:WAV audio;"
+    "audio/x-wav:wav:WAV audio;"
+    /* 3GPP */
+    "audio/3gpp:3gp,3gpp:3GPP audio;"
+    "video/3gpp:3gp,3gpp:3GPP video;"
+    /* 3GPP2 */
+    "audio/3gpp2:3g2,3gpp2:3GPP2 audio;"
+    "video/3gpp2:3g2,3gpp2:3GPP2 video;"
+    /* DIVX */
+    "video/divx:divx:DivX video;"
+    /* FLV */
+    "video/flv:flv:FLV video;"
+    "video/x-flv:flv:FLV video;"
+    /* Matroska */
+    "application/x-matroska:mkv:Matroska video;"
+    "video/x-matroska:mkv:Matroska video;"
+    "audio/x-matroska:mka:Matroska audio;"
+    /* XSPF */
+    "application/xspf+xml:xspf:Playlist xspf;"
+    /* M3U */
+    "audio/x-mpegurl:m3u:MPEG audio;"
+    /* Webm */
+    "video/webm:webm:WebM video;"
+    "audio/webm:webm:WebM audio;"
+    /* Real Media */
+    "application/vnd.rn-realmedia:rm:Real Media File;"
+    "audio/x-realaudio:ra:Real Media Audio;"
+    /* AMR */
+    "audio/amr:amr:AMR audio;"
+    /* FLAC */
+    "audio/x-flac:flac:FLAC audio;"
+    "audio/flac:flac:FLAC audio;"
+    ;
 
 /******************************************************************************
  * UNIX-only API calls
@@ -103,6 +177,15 @@ NPError NPP_GetValue( NPP instance, NPPVariable variable, void *value )
             }
             break;
         }
+#if defined(XP_MACOSX)
+        case NPPVpluginCoreAnimationLayer:
+        {
+            if( instance )
+                return p_plugin->get_root_layer(value);
+
+            break;
+        }
+#endif
 
         default:
             ;
@@ -111,7 +194,7 @@ NPError NPP_GetValue( NPP instance, NPPVariable variable, void *value )
 }
 
 /*
- * there is some confusion in gecko headers regarding definition of this API
+ * there is some confusion in NPAPI headers regarding definition of this API
  * NPPVariable is wrongly defined as NPNVariable, which sounds incorrect.
  */
 
@@ -120,121 +203,6 @@ NPError NPP_SetValue( NPP, NPNVariable, void * )
     return NPERR_GENERIC_ERROR;
 }
 
-/******************************************************************************
- * Mac-only API calls
- *****************************************************************************/
-#ifdef XP_MACOSX
-int16_t NPP_HandleEvent( NPP instance, void * event )
-{
-    static UInt32 lastMouseUp = 0;
-    if( instance == NULL )
-    {
-        return false;
-    }
-
-    VlcPluginBase *p_plugin = reinterpret_cast<VlcPluginBase *>(instance->pdata);
-    if( p_plugin == NULL )
-    {
-        return false;
-    }
-
-#ifndef __x86_64__
-    EventRecord *myEvent = (EventRecord*)event;
-
-    switch( myEvent->what )
-    {
-        case nullEvent:
-            return true;
-        case mouseDown:
-        {
-            if( (myEvent->when - lastMouseUp) < GetDblTime() )
-            {
-                /* double click */
-                p_plugin->toggle_fullscreen();
-            }
-            return true;
-        }
-        case mouseUp:
-            lastMouseUp = myEvent->when;
-            return true;
-        case keyUp:
-        case keyDown:
-        case autoKey:
-            return true;
-        case updateEvt:
-        {
-            const NPWindow& npwindow = p_plugin->getWindow();
-            if( npwindow.window )
-            {
-                bool hasVout = false;
-
-                if( p_plugin->playlist_isplaying() )
-                {
-                    hasVout = p_plugin->player_has_vout();
-#if 0
-                    if( hasVout )
-                    {
-                        libvlc_rectangle_t area;
-                        area.left = 0;
-                        area.top = 0;
-                        area.right = npwindow.width;
-                        area.bottom = npwindow.height;
-                        libvlc_video_redraw_rectangle(p_plugin->getMD(), &area, NULL);
-                    }
-#else
-#warning disabled code
-#endif
-                }
-
-                if( ! hasVout )
-                {
-                    /* draw the text from get_bg_text() */
-                    ForeColor(blackColor);
-                    PenMode( patCopy );
-
-                    /* seems that firefox forgets to set the following
-                     * on occasion (reload) */
-                    SetOrigin(((NP_Port *)npwindow.window)->portx,
-                              ((NP_Port *)npwindow.window)->porty);
-
-                    Rect rect;
-                    rect.left = 0;
-                    rect.top = 0;
-                    rect.right = npwindow.width;
-                    rect.bottom = npwindow.height;
-                    PaintRect( &rect );
-
-                    ForeColor(whiteColor);
-                    MoveTo( (npwindow.width-80)/ 2  , npwindow.height / 2 );
-                    if( !p_plugin->get_bg_text().empty() )
-                        DrawText( p_plugin->get_bg_text().c_str(), 0, p_plugin->get_bg_text().length() );
-                }
-            }
-            return true;
-        }
-        case activateEvt:
-            return false;
-        case NPEventType_GetFocusEvent:
-        case NPEventType_LoseFocusEvent:
-            return true;
-        case NPEventType_AdjustCursorEvent:
-            return false;
-        case NPEventType_MenuCommandEvent:
-            return false;
-        case NPEventType_ClippingChangedEvent:
-            return false;
-        case NPEventType_ScrollingBeginsEvent:
-            return true;
-        case NPEventType_ScrollingEndsEvent:
-            return true;
-        default:
-            ;
-    }
-#endif // __x86_64__
-    return false;
-}
-#endif /* XP_MACOSX */
-
 /******************************************************************************
  * General Plug-in Calls
  *****************************************************************************/
@@ -254,30 +222,64 @@ NPError NPP_Initialize( void )
     return NPERR_NO_ERROR;
 }
 
-#ifdef OJI
-jref NPP_GetJavaClass( void )
-{
-    return NULL;
-}
-#endif
-
 void NPP_Shutdown( void )
 {
     ;
 }
 
+static bool boolValue(const char *value) {
+    return ( !strcmp(value, "1") ||
+             !strcasecmp(value, "true") ||
+             !strcasecmp(value, "yes") );
+}
+
 NPError NPP_New( NPMIMEType, NPP instance,
                  NPuint16_t mode, NPint16_t argc,
                  char* argn[], char* argv[], NPSavedData* )
 {
     NPError status;
+    VlcPluginBase *p_plugin;
 
     if( instance == NULL )
     {
         return NPERR_INVALID_INSTANCE_ERROR;
     }
 
-    VlcPluginBase *p_plugin = new VlcPlugin( instance, mode );
+    bool windowless = false;
+    /* we need to tell whether the plugin will be windowless
+     * before it is instantiated */
+    for( int i = 0; i < argc; i++ )
+    {
+        if( !strcmp( argn[i], "windowless" ) )
+        {
+            windowless = boolValue(argv[i]);
+            break;
+        }
+    }
+
+    if( windowless )
+    {
+        printf( "Using Windowless mode\n" );
+        /* set windowless flag */
+        status = NPN_SetValue( instance, NPPVpluginWindowBool,
+                                       (void *)false);
+        if( NPERR_NO_ERROR != status )
+        {
+            return status;
+        }
+        status = NPN_SetValue( instance, NPPVpluginTransparentBool,
+                                       (void *)false);
+        if( NPERR_NO_ERROR != status )
+        {
+            return status;
+        }
+
+        p_plugin = new VlcWindowless( instance, mode );
+    }
+    else
+    {
+        p_plugin = new VlcPlugin( instance, mode );
+    }
     if( NULL == p_plugin )
     {
         return NPERR_OUT_OF_MEMORY_ERROR;
@@ -342,7 +344,7 @@ NPError NPP_SetWindow( NPP instance, NPWindow* window )
     /* retrieve current window */
     NPWindow& curr_window = p_plugin->getWindow();
 
-    if (window && window->window) {
+    if (window/* && window->window */) {
         if (!curr_window.window) {
             /* we've just been created */
             p_plugin->setWindow(*window);
@@ -392,7 +394,6 @@ NPError NPP_SetWindow( NPP instance, NPWindow* window )
             p_plugin->destroy_windows();
         }
     }
-
     return NPERR_NO_ERROR;
 }
 
@@ -546,3 +547,20 @@ void NPP_Print( NPP instance, NPPrint* printInfo )
         }
     }
 }
+
+int16_t NPP_HandleEvent( NPP instance, void * event )
+{
+    if( instance == NULL )
+    {
+        return false;
+    }
+
+    VlcPluginBase *p_plugin = reinterpret_cast<VlcPluginBase *>(instance->pdata);
+    if( p_plugin == NULL )
+    {
+        return false;
+    }
+
+    return p_plugin->handle_event(event);
+
+}
